// token.hh
// Abstract Token Class
// Kevin Puckett Albrecht
// Rev 14
// 20 March 2000 - 6 May 2001
//
// must define: GLOBAL_TOKEN_SIZE

#ifndef TOKEN_HH
#define TOKEN_HH

/////////////////////////
// DEFINES //////////////
/////////////////////////

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "value.hh"

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

enum TokenCode
{
  // dummy //
  codeDummy,

  // types //
  codeIdentifier, codeNumber, codeString,

  // control structures //
  codeEOF, codeEOL,

  // reserved words //
  codeIF, codeOR, codeIS,
  codeINT, codeRAT, codeSTR, codeEND, codeAND, codeNOT, codeVAR, codeFIN, codePUT,
  codeBOOL, codeFOUT, codeELSE, codeLOOP, codeGOTO, codeQUIT, codeSGET, codeFPUT, codePUTN,
  codeBLOCK, codeCONST, codeISNOT, codeSNEXT, codeSZERO, codeFPUTN, codeMRAND,
  codeRETURN, codeELSEIF, codeFCLOSE, 
  codeGETLINE,
  codeIMPLICIT, codeEXPLICIT, codeFGETLINE,

  // specials //
  codeCOLON, codeCONVERT, codeQUESTIONMARK,
  codeLEFTPAREN, codeRIGHTPAREN, codeCOMMA, codeDOT,
  codeEQUAL, codeNOTEQUAL, codeLT, codeGT, codeLTEQUAL, codeGTEQUAL,
  codePLUS, codeMINUS, codeTIMES, codeDIVIDE, codeAPPEND, codeANDSIGN,

  // special markers for LICode //
  codeLineMarker, codeShortMarker
};

char *gvsTokenStrings[] =
{
  // dummy //
  "<null>",

  // types //
  "<identifier>", "<number>", "<string>",

  // control structures //
  "<eof>", "<eol>",

  // reserved words //
  "if", "or", "is",
  "int", "rat", "str", "end", "and", "not", "var", "fin", "put"
  "bool", "fout", "else", "loop", "goto", "quit", "fput", "putn"
  "block", "const", "isnot", "snext", "szero", "fputn", "mrand",
  "return", "elseif", "fclose",
  "fgetline",
  "implicit", "explicit", "fgetline",

  // specials //
  ":", "<<", "?",
  "(", ")", ",", ".",
  "=", "<>", "<", ">", "<=", ">=",
  "+", "-", "*", "/", "&", "@",

  // special markers for LICode //
  "<line_marker>", "<short_marker>"
};

class Token
{
  public:
    TokenCode code;
    char buffer[GLOBAL_TOKEN_SIZE];

    Type type;    // the type in the node
    bool bConst;  // true if the node's contents are constant
    char *value;  // the value in the node, if type == tValue
    short marker; // the marker in the node, if type == tMarker or tSection

    void SetContent (Type aType, bool abConst, char *asValue, short anMarker = 0)
    {
      type = aType;
      bConst = abConst;

      //BUG: value may never be initialized// k_del (value);
      value = k_clone (asValue);

      marker = anMarker;
    }

    virtual void Get () = 0;
    virtual bool IsDelimiter () const = 0;
};

#endif